TITLE = "Title"
TITLE_EXPLAIN = "Give the issue an explicit title."
DESCRIPTION = "Description"
DESCRIPTION_EXPLAIN = "Detail the issue you have observed."
FEEDBACK_SAVE_BUTTON = "Save feedback"
FEEDBACK_SAVED = "Feedback saved & sent to the Data Science team."
FEEDBACK_SAVED_HTML = f'<p style="color:Green;">{FEEDBACK_SAVED}</p>'
FEEDBACK_NOT_SAVED = "Please specify a feedback title and a description."
FEEDBACK_NOT_SAVED_HTML = '<p style="color:Red;">Please specify a feedback title and a description.</p>'
